@@ -30,15 +30,16 @@
 
 typedef struct Set Set;
 
-Set *set_new(hash_func_t hash_func, compare_func_t compare_func);
+Set *set_new(const struct hash_ops *hash_ops);
 void set_free(Set* s);
 
 int set_put(Set *s, void *value);
 void *set_get(Set *s, void *value);
 bool set_contains(Set *s, void *value);
 
+int set_reserve(Set *s, unsigned entries_add);
+
 void *set_iterate(Set *s, Iterator *i);
-void *set_iterate_backwards(Set *s, Iterator *i);
 
 DEFINE_TRIVIAL_CLEANUP_FUNC(Set*, set_free);
 #define _cleanup_set_free_ _cleanup_(set_freep)
